<!--
SPDX-FileCopyrightText: 2020 2020 UMG MeDIC <marcel.parciak@med.uni-goettingen.de>

SPDX-License-Identifier: GPL-3.0-or-later
-->

Uploads a list of files to CDSTAR. 

# CDSTAR Upload Agent

This Agent complied to the [external REST-API](https://github.com/automaticmode/active_workflow/blob/master/docs/remote_agent_api.md) of [ActiveWorkflow](https://github.com/automaticmode/active_workflow). It allows to upload files from a source path to CDSTAR.

## API Caller Configurations

A caller of this agent has to comply to the remote agent API by ActiveWorkflow.

### Register

Registering this agent necessitates setting some options needed to work with Retrieve and Check.

* `source_directory` specifies a sub-path under the configured `base_directory` in which the agent will search for all supplied files when retrieving.
* `source_filemask` specifies a mask for files that will be applied when automatically searching for files under `source_directory`.
* `delete_after_upload` specifies whether uploaded files should be deleted.
* `mime_type` allows to set a specific mime_type to set in CDSTAR. If `None`, the autodetection method of CSTAR will be used.

For example, registering the agent may look like this:

```
{
    "source_directory": "/some/directory",
    "mime_type": "application/csv",
    "delete_after_upload": true
}
```

### Receive

The agent expects to receive a list of filenames that shall be uploaded in the format of the [File Mover Agent](https://gitlab.gwdg.de/medinf/umgmedic/active_workflow_agent_mover). Each filename is expected to be under `base_directory`/`source_directory`. The optional `source_filemask` is not applied to this list.

An exemplary message sent to the CDSTAR Upload Agent may look like this:

```
{
    "files_moved": [ "test.csv", "test2.csv" ]
}
```

Moreover, options may be overridden by a call to receive:

```
{
    "files_moved": [ "test.csv", "test2.csv" ],
    "options": {
        "source_directory": "/another/directory",
        "delete_after_upload": false
    }
}
```

### Check

The check method is fired by Active Workflow itself and has no need for any configuration. It is recommended to schedule the agent every 5 to 10 minutes to retrieve any logs and errors quickly.

The check method is the only one that supplies output messages. The output messages consist of an upload id (which is internally assigned and has no real meaning afterwards right now) and a archive id pointing to a CDSTAR archive.

Example message:

```
{
    "upload_id":"SgzNxarLwigi",
    "archive_id":"13a9a9577d4f"
}
```

## Service Configuration

Configuration can be done using environment variables. It does not matter whether done locally or given to docker.

* `source_directory`: Default folder that will be scanned for files. It may be overridden by the caller. Default: `"/tmp/source"`
* `source_filemask`: Default filemask for file matching. It may be overridden by the caller. Mind that this is python regex. Default `".*.csv"`
* `delete_after_upload`: Boolean to indicate whether uploaded files shall be deleted. It may be overridden by the caller. Default: `True`
* `mime_type`: MIME Type that shall be used for uploads to CDSTAR. It may be overridden by the caller. Default: `"applciation/csv"`
* `base_directory`: Default base directory which cannot be overridden with options or messages. Useful to limit the access of the application to certain folders. Default: `"/tmp/source"`
* `db_uri`: Path to a database (tested with postgres-12) in the [sqlalchemy notation](https://docs.sqlalchemy.org/en/13/core/engines.html#database-urls): `postgresql://<user>:<pass>@<host>:<port>/<db>`. Default: `"postgresql://uploader:test@localhost:5432/uploader"`
* `cdstar_uri`: The CDSTAR URL where the files shall be uploaded to including the version of the API. This will be passed directly to [pycdstar3](https://gitlab.gwdg.de/cdstar/pycdstar3) to establish a connection. Default: `"http://localhost:8082/v3"`
* `cdstar_user`: The user to authenticate with CDSTAR. Default: `"source"`
* `cdstar_pass`: The password to authenticate with CDSTAR. Default: `"something"`
* `cdstar_vault_target`: The vault that will be used to create new archives. Default: `"sources"`

To initialize the database, this agent makes use of [alembic](https://alembic.sqlalchemy.org). In its docker container, this is run automatically after 5 seconds using the `prestart.sh` shell script.